<script setup>
import { getCurrentInstance, ref, computed, reactive } from 'vue';
import { useRouter } from "vue-router";
import { get, post, postBlob, postForm } from '@/utils/request.js'
import { message, Modal } from 'ant-design-vue';
import { globalState, logout } from '@/config.js';
import EditView from './edit.vue'
import { formatMessage } from '@/utils/localeUtils.js';
import useTableSearch from '@/utils/tableUtils.jsx';
import MatPrint from '@/components/print/matPrint/index.vue';
import {
  DownOutlined,
  UploadOutlined,
  DownloadOutlined,
} from "@ant-design/icons-vue";

const context = getCurrentInstance()?.appContext.config.globalProperties;

const router = useRouter();

const TABLE_KEY = 'table-mat';
let currentPage = 1;
let pageSize = 10;
const searchInput = ref("")
const searchParam = ref({
  matnr: null,
  maktx: null,
  specs: null,
  model: null,
})
const editChild = ref(null)
const printChild = ref(null);

const state = reactive({
  selectedRowKeys: [],
  loading: false,
  columns: []
});

let tableData = ref([]);
getColumns();
getPage();

const {
  getColumnSearchProps,
  handleResizeColumn,
} = useTableSearch();


const hasSelected = computed(() => state.selectedRowKeys.length > 0);
const start = () => {
  state.loading = true;
  // ajax request after empty completing
  setTimeout(() => {
    state.loading = false;
    state.selectedRowKeys = [];
  }, 1000);
};
const onSelectChange = selectedRowKeys => {
  // console.log('selectedRowKeys changed: ', selectedRowKeys);
  state.selectedRowKeys = selectedRowKeys;
};

state.columns = [
  {
    title: '#ID',
    dataIndex: 'id',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('id'),
  },
  {
    title: formatMessage('db.man_mat.tag_id', '所属归类'),
    dataIndex: 'tagId$',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('tagId$'),
  },
  {
    title: formatMessage('db.man_mat.matnr', '商品编号'),
    dataIndex: 'matnr',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('matnr'),
  },
  {
    title: formatMessage('db.man_mat.maktx', '商品名称'),
    dataIndex: 'maktx',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('maktx'),
  },
  {
    title: formatMessage('db.man_mat.name', '别名'),
    dataIndex: 'name',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('name'),
  },
  {
    title: formatMessage('db.man_mat.specs', '规格'),
    dataIndex: 'specs',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('specs'),
  },
  {
    title: formatMessage('db.man_mat.model', '型号'),
    dataIndex: 'model',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('model'),
  },
  {
    title: formatMessage('db.man_mat.color', '颜色'),
    dataIndex: 'color',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('color'),
  },
  {
    title: formatMessage('db.man_mat.brand', '品牌'),
    dataIndex: 'brand',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('brand'),
  },
  {
    title: formatMessage('db.man_mat.unit', '单位'),
    dataIndex: 'unit',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('unit'),
  },
  {
    title: formatMessage('db.man_mat.price', '单价'),
    dataIndex: 'price',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('price'),
  },
  {
    title: formatMessage('db.man_mat.sku', 'sku'),
    dataIndex: 'sku',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('sku'),
  },
  {
    title: formatMessage('db.man_mat.units', '单位量'),
    dataIndex: 'units',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('units'),
  },
  {
    title: formatMessage('db.man_mat.barcode', '条码'),
    dataIndex: 'barcode',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('barcode'),
  },
  {
    title: formatMessage('db.man_mat.origin', '产地'),
    dataIndex: 'origin',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('origin'),
  },
  {
    title: formatMessage('db.man_mat.manu', '厂家'),
    dataIndex: 'manu',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('manu'),
  },
  {
    title: formatMessage('db.man_mat.manu_date', '生产日期'),
    dataIndex: 'manuDate',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('manuDate'),
  },
  {
    title: formatMessage('db.man_mat.item_num', '品项数'),
    dataIndex: 'itemNum',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('itemNum'),
  },
  {
    title: formatMessage('db.man_mat.weight', '重量'),
    dataIndex: 'weight',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('weight'),
  },
  {
    title: formatMessage('db.man_mat.length', '长度'),
    dataIndex: 'length',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('length'),
  },
  {
    title: formatMessage('db.man_mat.volume', '体积'),
    dataIndex: 'volume',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('volume'),
  },
  {
    title: formatMessage('db.man_mat.three_code', '三方编码'),
    dataIndex: 'threeCode',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('threeCode'),
  },
  {
    title: formatMessage('db.man_mat.supp', '供应商'),
    dataIndex: 'supp',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('supp'),
  },
  {
    title: formatMessage('db.man_mat.supp_code', '供应商编码'),
    dataIndex: 'suppCode',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('suppCode'),
  },
  {
    title: formatMessage('db.man_mat.dead_time', '保质期'),
    dataIndex: 'deadTime',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('deadTime'),
  },
  {
    title: formatMessage('db.man_mat.safeStockLimit', '安全库存上限'),
    dataIndex: 'safeStockLimit',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('safeStockLimit'),
  },
  {
    title: formatMessage('db.man_mat.safeStockMinimum', '安全库存下限'),
    dataIndex: 'safeStockMinimum',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('safeStockMinimum'),
  },
  {
    title: formatMessage('db.man_mat.validity', '有效期'),
    dataIndex: 'validity',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('validity'),
  },

  {
    title: formatMessage('db.man_mat.uuid', '编号'),
    dataIndex: 'uuid',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('uuid'),
  },
  {
    title: formatMessage('db.man_mat.flag', '标识'),
    dataIndex: 'flag',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('flag'),
  },
  {
    title: formatMessage('db.man_mat.sort', '排序'),
    dataIndex: 'sort',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('sort'),
  },
  {
    title: formatMessage('db.man_mat.status', '状态'),
    dataIndex: 'status$',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('status$'),
  },
  // {
  //   title: formatMessage('db.man_mat.create_time', '添加时间'),
  //   dataIndex: 'createTime$',
  //   width: 140,
  //   ellipsis: true,
  // },
  // {
  //   title: formatMessage('db.man_mat.create_by', '添加人员'),
  //   dataIndex: 'createBy$',
  //   width: 140,
  //   ellipsis: true,
  // },
  // {
  //   title: formatMessage('db.man_mat.update_time', '修改时间'),
  //   dataIndex: 'updateTime$',
  //   width: 140,
  //   ellipsis: true,
  // },
  // {
  //   title: formatMessage('db.man_mat.update_by', '修改人员'),
  //   dataIndex: 'updateBy$',
  //   width: 140,
  //   ellipsis: true,
  // },
  {
    title: formatMessage('db.man_mat.memo', '备注'),
    dataIndex: 'memo',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('memo'),
  },

];

//加载扩展字段
async function getColumns() {
  let fieldResp = await post('/api/matField/list', {
    fieldType: 0
  })
  let fieldResult = fieldResp.data;
  let tmp = state.columns;
  if (fieldResult.code == 200) {
    let data = fieldResult.data;

    data.forEach((item) => {
      tmp.push({
        title: formatMessage(item.language, item.describe),
        name: item.name,
        dataIndex: item.name,
        key: item.name,
        width: 140,
      })
    })

    tmp.push({
      title: formatMessage('common.operation', '操作'),
      name: 'oper',
      dataIndex: 'oper',
      key: 'oper',
      width: 140,
      fixed: 'right',
    })

    state.columns = tmp;
  } else if (result.code === 401) {
    message.error(result.msg);
    logout()
  } else {
    message.error(result.msg);
  }
}

async function getPage() {
  state.loading = true;
  post('/api/mat/page', {
    current: currentPage,
    pageSize: pageSize,
    condition: searchInput.value,
    _param: searchParam.value,
  }).then((resp) => {
    let result = resp.data;
    if (result.code == 200) {
      let data = result.data;
      tableData.value = data;

      state.loading = false;
    } else if (result.code === 401) {
      message.error(result.msg);
      logout()
    } else {
      message.error(result.msg);
    }
  })
}

const handleEdit = (item) => {
  editChild.value.open = true;
  editChild.value.formData = item == null ? editChild.value.initFormData : JSON.parse(JSON.stringify(item));
  editChild.value.isSave = item == null;
}

const handleDel = (rows) => {
  Modal.confirm({
    title: formatMessage('page.delete', '删除'),
    content: formatMessage('page.delete.confirm', '确定删除该项吗？'),
    maskClosable: true,
    onOk: async () => {
      const hide = message.loading(formatMessage('common.loading', '请求中'));
      try {
        post('/api/mat/remove/' + rows.map((row) => row.id).join(','), {}).then(resp => {
          let result = resp.data;
          if (result.code === 200) {
            message.success(result.msg);
          } else {
            message.error(result.msg);
          }
          getPage()
          hide()
        })
      } catch (error) {
        message.error(formatMessage('common.fail', '请求失败'));
      }
    },
  });
}

const handleExport = async (intl) => {
  postBlob('/api/mat/export', {
    condition: searchInput.value,
    _param: searchParam.value,
  }).then(result => {
    const blob = new Blob([result.data], { type: 'application/vnd.ms-excel' });
    window.location.href = window.URL.createObjectURL(blob);
    return true;
  })
};

const onSearch = () => {
  // console.log('search');
  getPage()
}

const onPageChange = (page, size) => {
  currentPage = page;
  pageSize = size;
  getPage();
}

const showTotalPage = (total, range) => {
  return formatMessage('page.total.head', '总计') + total + formatMessage('page.total.footer', '条')
}

function handleTableReload(value) {
  getPage()
}

const matFieldList = ref(null);
MatFieldQuery();

function MatFieldQuery() {
  post('/api/matField/list', {}).then(resp => {
    let result = resp.data;
    matFieldList.value = result.data;
  })
}

const handleSyncMatClick = (e) => {
  if (e.key == 'import') {
    importTemplate()
  } else if (e.key == 'export') {
    exportTemplate()
  }
}

const importTemplate = () => {
  //导入模板
}

const exportTemplate = () => {
  //模板导出
  postBlob('/api/mat/exportTemplate', {}).then(result => {
    const blob = new Blob([result.data], { type: 'application/vnd.ms-excel' });
    window.location.href = window.URL.createObjectURL(blob);
    return true;
  })
}

const fileList = ref([]);
const handleUploadChange = info => {
  if (info.file.status !== 'uploading') {
    // console.log(info.file, info.fileList);
  }
  if (info.file.status === 'done') {
    let result = info.file.response;
    if (result.code == 200) {
      message.success(`${info.file.name} ${formatMessage('page.upload.success', '上传成功')}`);
      getPage()
    } else {
      message.error(result.msg);
    }
  } else if (info.file.status === 'error') {
    message.error(`${info.file.name} file upload failed.`);
  }
};

const handlePrint = () => {
  if (state.selectedRowKeys.length == 0) {
    message.warning(formatMessage('common.select', '请选择'));
    return;
  }
  let data = tableData.value.records;
  let tmp = [];
  data.forEach((item) => {
    if (state.selectedRowKeys.indexOf(item.id) != -1) {
      tmp.push(item);
    }
  })

  printChild.value.printData = tmp;
  printChild.value.open = true;
}

</script>

<script>
export default {
  name: '商品档案'
}
</script>

<template>
  <div>
    <EditView ref="editChild" @tableReload="handleTableReload" />
    <div class="table-header">
      <div>
        <a-input v-model:value="searchParam.matnr" :placeholder="formatMessage('page.mat.matnr.input', '请输入商品编号')"
          style="width: 140px;margin-right: 10px;" />
        <a-input v-model:value="searchParam.maktx" :placeholder="formatMessage('page.mat.matnr.input', '请输入商品名称')"
          style="width: 140px;margin-right: 10px;" />
        <a-input v-model:value="searchParam.specs" :placeholder="formatMessage('page.mat.matnr.input', '请输入规格')"
          style="width: 140px;margin-right: 10px;" />
        <a-input v-model:value="searchParam.model" :placeholder="formatMessage('page.mat.matnr.input', '请输入型号')"
          style="width: 140px;margin-right: 10px;" />

        <a-input-search v-model:value="searchInput" :placeholder="formatMessage('page.input', '请输入')"
          style="width: 200px;" @search="onSearch" />
      </div>
      <div class="table-header-right">
        <a-dropdown>
          <template #overlay>
            <a-menu @click="handleSyncMatClick">
              <a-menu-item key="import">
                <a-upload v-model:file-list="fileList" name="file" action="/api/mat/upload" @change="handleUploadChange"
                  :showUploadList="false" :headers="{
                    Authorization: globalState.token
                  }">
                  <UploadOutlined />
                  {{ formatMessage('page.mat.import', '商品导入') }}
                </a-upload>
              </a-menu-item>

              <a-menu-item key="export">
                <DownloadOutlined />
                {{ formatMessage('page.mat.export.template', '导出模板') }}
              </a-menu-item>
            </a-menu>
          </template>
          <a-button>
            {{ formatMessage('page.mat.sync', '数据同步') }}
            <DownOutlined />
          </a-button>
        </a-dropdown>
        <a-button @click="handlePrint()" type="primary">
          {{ formatMessage('db.man_zpallet_barcode.batchPrint', '批量打印') }}
        </a-button>
        <a-button @click="handleEdit(null)" type="primary">{{ formatMessage('page.add', '添加') }}</a-button>
        <a-button @click="handleExport">{{ formatMessage('page.export', '导出') }}</a-button>
      </div>
    </div>
    <a-table :row-selection="{ selectedRowKeys: state.selectedRowKeys, onChange: onSelectChange }"
      :data-source="tableData.records" :defaultExpandAllRows="false" :key="TABLE_KEY" rowKey="id"
      :pagination="{ total: tableData.total, showTotal: showTotalPage, onChange: onPageChange }"
      :scroll="{ y: 768, scrollToFirstRowOnChange: true }" :columns="state.columns" @resizeColumn="handleResizeColumn"
      :loading="state.loading">
      <template #bodyCell="{ column, text, record }">
        <template v-if="column.dataIndex === 'oper'">
          <div style="display: flex;justify-content: space-evenly;">
            <a-button type="link" primary @click="handleEdit(record)">{{
              formatMessage('page.edit', '编辑')
            }}
            </a-button>
            <a-button type="link" danger @click="handleDel([record])">
              {{ formatMessage('page.delete', '删除') }}
            </a-button>
          </div>
        </template>
      </template>
    </a-table>
  </div>

  <MatPrint ref="printChild" />
</template>

<style></style>
